@@ -2,7 +2,7 @@ package Net::Twitter;
 use Moose;
 use Carp;
 use Net::Twitter::Core;
-use Digest::SHA1 qw/sha1_hex/;
+use Digest::SHA qw/sha1_hex/;
 
 use namespace::autoclean;
 
@@ -12,7 +12,7 @@ has '_trait_namespace' => (
 );
 
 # use *all* digits for fBSD ports
-our $VERSION = '3.16000_1';
+our $VERSION = '3.18000_00';
 
 $VERSION = eval $VERSION; # numify for warning-free dev releases
 
@@ -68,7 +68,7 @@ sub _create_anon_class {
         superclasses => $superclasses,
         roles        => $traits,
         methods      => { meta => sub { $meta }, isa => \&_isa },
-        cache        => 1,
+        cache        => 0,
         package      => $package,
     );
     $meta->make_immutable(inline_constructor => $immutable);
@@ -76,22 +76,28 @@ sub _create_anon_class {
     return $meta;
 }
 
-sub _name_for_anon_class {
-    my @t = @{$_[0]};
+{
+    my $serial_number = 0;
+    my %serial_for_params;
 
-    my @comps;
-    while ( @t ) {
-        my $t = shift @t;
-        if ( ref $t[0] eq 'HASH' ) {
-            my $r = shift @t;
-            my $sig = sha1_hex(JSON::Any->to_json($r));
-            $t .= "_$sig";
+    sub _name_for_anon_class {
+        my @t = @{$_[0]};
+
+        my @comps;
+        while ( @t ) {
+            my $t = shift @t;
+            if ( ref $t[0] eq 'HASH' ) {
+                my $params = shift @t;
+                my $sig = sha1_hex(JSON::Any->to_json($params));
+                my $sn  = $serial_for_params{$sig} ||= ++$serial_number;
+                $t .= "_$sn";
+            }
+            $t =~ s/(?:::|\W)/_/g;
+            push @comps, $t;
         }
-        $t =~ s/(?:::|\W)/_/g;
-        push @comps, $t;
-    }
 
-    return join '', __PACKAGE__, '::',  join '__', 'with', sort @comps;
+        return __PACKAGE__ . '::' .  join '__', 'with', sort @comps;
+    }
 }
 
 sub new {
