@@ -6,6 +6,8 @@ use URI;
 
 requires qw/ua username password credentials/;
 
+with 'Net::Twitter::Role::API::Upload';
+
 has apiurl          => ( isa => 'Str', is => 'ro', default => 'http://api.twitter.com/1'  );
 has apihost         => ( isa => 'Str', is => 'ro', lazy => 1, builder => '_build_apihost' );
 has apirealm        => ( isa => 'Str', is => 'ro', default => 'Twitter API'               );
@@ -27,6 +29,7 @@ authenticate 1;
 our $DATETIME_PARSER = DateTime::Format::Strptime->new(pattern => '%a %b %d %T %z %Y');
 datetime_parser $DATETIME_PARSER;
 
+
 twitter_api_method public_timeline => (
     description => <<'EOT',
 Returns the 20 most recent statuses from non-protected users who have
@@ -127,6 +130,7 @@ retweeted by others.
 );
 
 twitter_api_method friends_timeline => (
+    deprecated  => 1,
     description => <<'',
 Returns the 20 most recent statuses posted by the authenticating user
 and that user's friends. This is the equivalent of /home on the Web.
@@ -290,6 +294,7 @@ EOT
 );
 
 twitter_api_method followers => (
+    deprecated  => 1,
     description => <<'EOT',
 Returns a reference to an array of the user's followers.  If C<id>, C<user_id>,
 or C<screen_name> is not specified, the followers of the authenticating user are
@@ -332,6 +337,30 @@ authenticated to request the page of a protected user.
     returns  => 'ExtendedUser',
 );
 
+twitter_api_method contributees => (
+    path        => 'users/contributees',
+    method      => 'GET',
+    params      => [qw/user_id screen_name include_entities skip_satus/],
+    required    => [],
+    booleans    => [qw/include_entities skip_satus/],
+    returns     => 'ArrayRef[User]',
+    description => <<'',
+Returns an array of users that the specified user can contribute to.
+
+);
+
+twitter_api_method contributors => (
+    path        => 'users/contributors',
+    method      => 'GET',
+    params      => [qw/user_id screen_name include_entities skip_satus/],
+    required    => [],
+    booleans    => [qw/include_entities skip_satus/],
+    returns     => 'ArrayRef[User]',
+    description => <<'',
+Returns an array of users who can contribute to the specified account.
+
+);
+
 twitter_api_method direct_messages => (
     description => <<'',
 Returns a list of the 20 most recent direct messages sent to the authenticating
@@ -735,7 +764,7 @@ Returns the un-blocked user when successful.
 
     path     => 'blocks/destroy/:id',
     method   => 'POST',
-    params   => [qw/id user_idscreen_name/],
+    params   => [qw/id user_id screen_name/],
     booleans => [qw/include_entities/],
     required => [qw/id/],
     returns  => 'BasicUser',
@@ -804,6 +833,35 @@ maintenance window is scheduled.
     returns  => 'Str',
 );
 
+twitter_api_method get_configuration => (
+    path        => 'help/configuration',
+    method      => 'GET',
+    params      => [],
+    required    => [],
+    returns     => 'HashRef',
+    description => <<'EOT',
+Returns the current configuration used by Twitter including twitter.com slugs
+which are not usernames, maximum photo resolutions, and t.co URL lengths.
+
+It is recommended applications request this endpoint when they are loaded, but
+no more than once a day.
+EOT
+
+);
+
+twitter_api_method get_languages => (
+    path        => 'help/languages',
+    method      => 'GET',
+    params      => [],
+    required    => [],
+    returns     => 'ArrayRef[Lanugage]',
+    description => <<'',
+Returns the list of languages supported by Twitter along with their ISO 639-1
+code. The ISO 639-1 code is the two letter value to use if you include lang
+with any of your requests.
+
+);
+
 twitter_api_method saved_searches => (
     description => <<'',
 Returns the authenticated user's saved search queries.
@@ -826,17 +884,6 @@ Retrieve the data for a saved search, by C<id>, owned by the authenticating user
     returns  => 'SavedSearch',
 );
 
-twitter_api_method show_saved_search => (
-    description => <<'',
-Retrieve the data for a saved search, by ID, owned by the authenticating user.
-
-    path     => 'saved_searches/show/:id',
-    method   => 'GET',
-    params   => [qw/id/],
-    required => [qw/id/],
-    returns  => 'SavedSearch',
-);
-
 twitter_api_method create_saved_search => (
     description => <<'',
 Creates a saved search for the authenticated user.
@@ -1047,6 +1094,71 @@ Returns details of a place returned from the C<reverse_geocode> method.
 EOT
 );
 
+twitter_api_method geo_search => (
+    path        => 'geo/search',
+    method      => 'GET',
+    params      => [qw/
+        lat long query ip granularity accuracy max_results
+        contained_within attribute:street_address callback
+    /],
+    required    => [],
+    returns     => 'HashRef',
+    description => <<'EOT',
+Search for places that can be attached to a statuses/update. Given a latitude
+and a longitude pair, an IP address, or a name, this request will return a list
+of all the valid places that can be used as the place_id when updating a
+status.
+
+Conceptually, a query can be made from the user's location, retrieve a list of
+places, have the user validate the location he or she is at, and then send the
+ID of this location with a call to statuses/update.
+
+This is the recommended method to use find places that can be attached to
+statuses/update. Unlike geo/reverse_geocode which provides raw data access,
+this endpoint can potentially re-order places with regards to the user who
+is authenticated. This approach is also preferred for interactive place
+matching with the user.
+EOT
+
+);
+
+twitter_api_method similar_places => (
+    path        => 'geo/similar_places',
+    method      => 'GET',
+    params      => [qw/lat long name contained_within attribute:street_address callback/],
+    required    => [qw/lat long name/],
+    returns     => 'HashRef',
+    description => <<'EOT',
+Locates places near the given coordinates which are similar in name.
+
+Conceptually you would use this method to get a list of known places to choose
+from first. Then, if the desired place doesn't exist, make a request to
+C<add_place> to create a new one.
+
+The token contained in the response is the token needed to be able to create a
+new place.
+EOT
+
+);
+
+twitter_api_method add_place => (
+    path        => 'geo/place',
+    method      => 'POST',
+    params      => [qw/name contained_within token lat long attribute:street_address callback/],
+    required    => [qw/name contained_within token lat long/],
+    returns     => 'Place',
+    description => <<'EOT',
+Creates a new place object at the given latitude and longitude.
+
+Before creating a place you need to query C<similar_places> with the latitude,
+longitude and name of the place you wish to create. The query will return an
+array of places which are similar to the one you wish to create, and a token.
+If the place you wish to create isn't in the returned array you can use the
+token with this method to create a new one.
+EOT
+
+);
+
 twitter_api_method lookup_users => (
     path => 'users/lookup',
     method => 'GET',
@@ -1128,7 +1240,7 @@ request.
 
 );
 
-# new in 3.16000_1 2010-10-19
+# new in 3.17001 2010-10-19
 
 twitter_api_method account_totals => (
     path        => 'account/totals',
@@ -1281,6 +1393,277 @@ response. This method is only available to users who have access to
 
 );
 
+### Lists ###
+
+twitter_api_method list_subscriptions => (
+    path        => 'lists/all',
+    method      => 'GET',
+    params      => [qw/user_id screen_name/],
+    required    => [],
+    returns     => 'ArrayRef[List]',
+    description => <<'',
+Returns all lists the authenticating or specified user subscribes to, including
+their own. The user is specified using the user_id or screen_name parameters.
+If no user is given, the authenticating user is used.
+
+);
+
+twitter_api_method list_statuses => (
+    path        => 'lists/statuses',
+    method      => 'GET',
+    params      => [qw/
+        list_id slug owner_screen_name owner_id since_id max_id per_page page
+        include_entities include_rts
+    /],
+    required    => [],
+    booleans    => [qw/include_entities include_rts/],
+    returns     => 'ArrayRef[Status]',
+    description => <<'',
+Returns tweet timeline for members of the specified list. Historically,
+retweets were not available in list timeline responses but you can now use the
+include_rts=true parameter to additionally receive retweet objects.
+
+);
+
+twitter_api_method delete_list_member => (
+    path        => 'lists/members/destroy',
+    method      => 'POST',
+    params      => [qw/list_id slug user_id screen_name owner_screen_name owner_id/],
+    required    => [],
+    returns     => 'User',
+    aliases     => [qw/remove_list_member/],
+    description => <<'',
+Removes the specified member from the list. The authenticated user must be the
+list's owner to remove members from the list.
+
+);
+
+twitter_api_method list_memberships => (
+    path        => 'lists/memberships',
+    method      => 'GET',
+    params      => [qw/user_id screen_name cursor filter_to_owned_lists/],
+    required    => [],
+    booleans    => [qw/filter_to_owned_lists/],
+    returns     => 'Hashref',
+    description => <<'',
+Returns the lists the specified user has been added to. If user_id or
+screen_name are not provided the memberships for the authenticating user are
+returned.
+
+);
+
+twitter_api_method list_subscribers => (
+    path        => 'lists/subscribers',
+    method      => 'GET',
+    params      => [qw/list_id slug owner_screen_name owner_id cursor include_entities skip_status/],
+    required    => [],
+    booleans    => [qw/include_entities skip_status/],
+    returns     => 'Hashref',
+    description => <<'',
+Returns the subscribers of the specified list. Private list subscribers will
+only be shown if the authenticated user owns the specified list.
+
+);
+
+twitter_api_method subscribe_list => (
+    path        => 'lists/subscribers/create',
+    method      => 'POST',
+    params      => [qw/owner_screen_name owner_id list_id slug/],
+    required    => [],
+    returns     => 'List',
+    description => <<'',
+Subscribes the authenticated user to the specified list.
+
+);
+
+twitter_api_method is_list_subscriber => (
+    path        => 'lists/subscribers/show',
+    method      => 'GET',
+    params      => [qw/
+        owner_screen_name owner_id list_id slug user_id screen_name
+        include_entities skip_status
+    /],
+    required    => [],
+    booleans    => [qw/include_entities skip_status/],
+    returns     => 'Maybe[User]',
+    aliases     => [qw/is_subscribed_list/],
+    description => <<'',
+Check if the specified user is a subscriber of the specified list. Returns the
+user or undef.
+
+);
+
+around [qw/is_list_subscriber is_subscribed_list/] => sub {
+    my $orig = shift;
+    my $self = shift;
+
+    $self->_user_or_undef($orig, 'subscriber', @_);
+};
+
+twitter_api_method unsubscribe_list => (
+    path        => 'lists/subscribers/destroy',
+    method      => 'POST',
+    params      => [qw/list_id slug owner_screen_name owner_id/],
+    required    => [],
+    returns     => 'List',
+    description => <<'',
+Unsubscribes the authenticated user from the specified list.
+
+);
+
+twitter_api_method members_create_all => (
+    path        => 'lists/members/create_all',
+    method      => 'POST',
+    params      => [qw/list_id slug owner_screen_name owner_id/],
+    required    => [],
+    returns     => 'List',
+    aliases     => [qw/add_list_members/],
+    description => <<'',
+Adds multiple members to a list, by specifying a reference to an array or a
+comma-separated list of member ids or screen names. The authenticated user must
+own the list to be able to add members to it. Note that lists can't have more
+than 500 members, and you are limited to adding up to 100 members to a list at
+a time with this method.
+
+);
+
+twitter_api_method is_list_member => (
+    path        => 'lists/members/show',
+    method      => 'GET',
+    params      => [qw/
+        owner_screen_name owner_id list_id slug user_id screen_name
+        include_entities skip_status
+    /],
+    required    => [],
+    booleans    => [qw/include_entities skip_status/],
+    returns     => 'Maybe[User]',
+    description => <<'',
+Check if the specified user is a member of the specified list. Returns the user or undef.
+
+);
+
+around is_list_member => sub {
+    my $orig = shift;
+    my $self = shift;
+
+    $self->_user_or_undef($orig, 'member', @_);
+};
+
+twitter_api_method list_members => (
+    path        => 'lists/members',
+    method      => 'GET',
+    params      => [qw/
+        list_id slug owner_screen_name owner_id cursor
+        include_entities skip_status
+    /],
+    required    => [],
+    booleans    => [qw/include_entities skip_status/],
+    returns     => 'Hashref',
+    description => <<'',
+Returns the members of the specified list. Private list members will only be
+shown if the authenticated user owns the specified list.
+
+);
+
+twitter_api_method add_list_member => (
+    path        => 'lists/members/create',
+    method      => 'POST',
+    params      => [qw/list_id slug user_id screen_name owner_screen_name owner_id/],
+    required    => [],
+    returns     => 'User',
+    description => <<'',
+Add a member to a list. The authenticated user must own the list to be able to
+add members to it. Note that lists can't have more than 500 members.
+
+);
+
+twitter_api_method delete_list => (
+    path        => 'lists/destroy',
+    method      => 'POST',
+    params      => [qw/owner_screen_name owner_id list_id slug/],
+    required    => [],
+    returns     => 'List',
+    description => <<'',
+Deletes the specified list. The authenticated user must own the list to be able
+to destroy it.
+
+);
+
+twitter_api_method update_list => (
+    path        => 'lists/update',
+    method      => 'POST',
+    params      => [qw/list_id slug name mode description owner_screen_name owner_id/],
+    required    => [],
+    returns     => 'List',
+    description => <<'',
+Updates the specified list. The authenticated user must own the list to be able
+to update it.
+
+);
+
+twitter_api_method create_list => (
+    path        => 'lists/create',
+    method      => 'POST',
+    params      => [qw/list_id slug name mode description owner_screen_name owner_id/],
+    required    => [],
+    returns     => 'List',
+    description => <<'',
+Creates a new list for the authenticated user. Note that you can't create more
+than 20 lists per account.
+
+);
+
+twitter_api_method get_lists => (
+    path        => 'lists',
+    method      => 'GET',
+    params      => [qw/user_id screen_name cursor/],
+    required    => [],
+    returns     => 'Hashref',
+    aliases     => [qw/list_lists/],
+    description => <<'',
+Returns the lists of the specified (or authenticated) user. Private lists will
+be included if the authenticated user is the same as the user whose lists are
+being returned.
+
+);
+
+twitter_api_method get_list => (
+    path        => 'lists/show',
+    method      => 'GET',
+    params      => [qw/list_id slug owner_screen_name owner_id/],
+    required    => [],
+    returns     => 'List',
+    description => <<'',
+Returns the specified list. Private lists will only be shown if the
+authenticated user owns the specified list.
+
+);
+
+### Legal ###
+
+twitter_api_method get_privacy_policy => (
+    path        => 'legal/privacy',
+    method      => 'GET',
+    params      => [],
+    required    => [],
+    returns     => 'HashRef',
+    description => <<'',
+Returns Twitter's privacy policy.
+
+);
+
+twitter_api_method get_tos => (
+    path        => 'legal/tos',
+    method      => 'GET',
+    params      => [],
+    required    => [],
+    returns     => 'HashRef',
+    description => <<'',
+Returns the Twitter Terms of Service. These are not the same as the Developer
+Rules of the Road.
+
+);
+
 1;
 
 __END__
